// PIC16F1827 Configuration Bit Settings

// 'C' source line config statements

#include <xc.h>

// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

// CONFIG1
#pragma config FOSC = INTOSC    // Oscillator Selection (INTOSC oscillator: I/O function on CLKIN pin)
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
#pragma config PWRTE = ON       // Power-up Timer Enable (PWRT enabled)
#pragma config MCLRE = OFF      // MCLR Pin Function Select (MCLR/VPP pin function is digital input)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config CPD = OFF        // Data Memory Code Protection (Data memory code protection is disabled)
#pragma config BOREN = ON       // Brown-out Reset Enable (Brown-out Reset enabled)
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)
#pragma config IESO = OFF       // Internal/External Switchover (Internal/External Switchover mode is disabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor is disabled)

// CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
#pragma config PLLEN = OFF      // PLL Enable (4x PLL disabled)
#pragma config STVREN = ON      // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will cause a Reset)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config LVP = OFF        // Low-Voltage Programming Enable (High-voltage on MCLR/VPP must be used for programming)

void tmrcnt();

bit secflg;//時間経過チェック用フラグ
unsigned char seccnt;//時間カウンタ

//割り込み処理
void interrupt intr(void) {
    TMR2IF = 0;//Timer2割り込みクリア
    tmrcnt();//時間カウント処理へ
}

//時間カウント処理
void tmrcnt(void){
    seccnt--;//割り込みが掛かる度に１つ減らす
    if (seccnt == 0){//0になれば指定時間が経過したことになる
        seccnt = 20;//カウンタを初期化
        secflg = 1;//指定時間経過
    }
}

//メイン処理
main(){

    OSCCON = 0b01110010;//内部クロック8MHz
    ANSELA = 0b00000000;//全てデジタルI/O
    ANSELB = 0b00000000;//全てデジタルI/O
    TRISA  = 0b00000000;//全て出力(RA5は入力のみ)
    TRISB  = 0b00000000;//全て出力

    INTCONbits.GIE = 1;//Global Interrupt Enable bit
    INTCONbits.PEIE = 1;//Peripheral Interrupt Enable bit



    //Timer2割り込み関連の設定
    PIE1bits.TMR2IE = 1;//TMR2 to PR2 match interrupt
    T2CONbits.T2OUTPS = 0b0100;//Timer2 Postscaler Select bits (1:5)
    T2CONbits.TMR2ON = 0b1;//Timer2 On bit
    T2CONbits.T2CKPS = 0b01;//Timer2-type Clock Prescale Select bits (4)
    TMR2 = 0x00;//Timer2 Module register
    PR2 = 25;//Timer2 Period register



    //PWM Period 500Hz 500 = 1/( (249 + 1) x 4 / 8000000 x 16 )
    //Set CCP4
    PR4 = 249;//Timer4 Period register
    CCP4CONbits.CCP4M2 = 1;
    CCP4CONbits.CCP4M3 = 1;//PWM mode
    //Duty Cycle
    CCPR4L = 0b01000000;//
    CCP4CONbits.DC4B = 0b00;//PWM Duty Cycle Least Significant bits

    CCPTMRSbits.C4TSEL = 0x01;//CCP4 is based off Timer 4 in PWM Mode

    PIR3bits.TMR4IF = 0;//TMR4 to PR4 Match Interrupt Flag bit

    T4CONbits.T4CKPS = 0b10;//Timer4-type Clock Prescale Select bits (16)
    T4CONbits.TMR4ON = 0b1;//Timer4 On bit
 
    //一定時間毎にCCPR4Lを減らす
    seccnt = 20;//8000000/4/16/25/10/2 = 250Hz
                //１秒間に8000000クロック発生(8MHz)する設定にしてある
                //４つ数えるごとに１つプリスケーラに送られる
                //プリスケーラ(TxCKPS)は16回数えるとタイマー２に１つ送る
                //タイマー２は25(PR2)と値を比較して等しいとポストスケーラに１つ送る
                //ポストスケーラー(TxOUTPS)は10回数えると割り込みを発生させる
                //以上を100回行うと8000000数えたことになる
    secflg = 0;//指定した時間が経過すると１

    while(1){
        if(secflg){//もし指定した時間が経過してたら
            secflg = 0;//次の指定時間までは０のまま
                CCPR4L--;
                if(CCPR4L==0){
                    RA0 ^= 1;//RA3のLEDを点滅(ビットを反転)
                }
        }
    }
}
